import os

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": "netfields",
    }
}

INSTALLED_APPS = (
    "django.contrib.postgres",
    "netfields",
    "test",
)

MIDDLEWARE_CLASSES = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
)

SECRET_KEY = "notimportant"

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"



from ipaddress import ip_address, ip_interface, ip_network

from netaddr import EUI
from netaddr.core import AddrFormatError
from rest_framework import serializers

from netfields.mac import mac_unix_common, mac_eui64
from netfields import fields


class InetAddressField(serializers.Field):
    default_error_messages = {
        'invalid': 'Invalid IP address.'
    }

    def __init__(self, store_prefix=True, *args, **kwargs):
        self.store_prefix = store_prefix
        super(InetAddressField, self).__init__(*args, **kwargs)

    def to_representation(self, value):
        if value is None:
            return value
        return str(value)

    def to_internal_value(self, data):
        if data is None:
            return data
        try:
            if self.store_prefix:
                return ip_interface(data)
            else:
                return ip_address(data)
        except ValueError:
            self.fail('invalid')


class CidrAddressField(serializers.Field):
    default_error_messages = {
        'invalid': 'Invalid CIDR address.',
        'network': 'Must be a network address.',
    }

    def to_representation(self, value):
        if value is None:
            return value
        return str(value)

    def to_internal_value(self, data):
        if data is None:
            return data
        try:
            return ip_network(data)
        except ValueError as e:
            if 'has host bits' in e.args[0]:
                self.fail('network')
            self.fail('invalid')


class MACAddressField(serializers.Field):
    default_error_messages = {
        'invalid': 'Invalid MAC address.'
    }

    def to_representation(self, value):
        if value is None:
            return value
        return str(value)

    def to_internal_value(self, data):
        if data is None:
            return data
        try:
            return EUI(data, version=48, dialect=mac_unix_common)
        except (AddrFormatError, IndexError, TypeError):
            self.fail('invalid')


class MACAddress8Field(serializers.Field):
    default_error_messages = {
        'invalid': 'Invalid MAC address 8.'
    }

    def to_representation(self, value):
        if value is None:
            return value
        return str(value)

    def to_internal_value(self, data):
        if data is None:
            return data
        try:
            mac = EUI(data, dialect=mac_eui64)
            if mac.version == 64:
                return mac
            mac = mac.eui64()
            mac.dialect = mac_eui64
            return mac
        except (AddrFormatError, IndexError, TypeError):
            self.fail('invalid')


class NetModelSerializer(serializers.ModelSerializer):
    pass


NetModelSerializer.serializer_field_mapping[fields.InetAddressField] = InetAddressField
NetModelSerializer.serializer_field_mapping[fields.CidrAddressField] = CidrAddressField
NetModelSerializer.serializer_field_mapping[fields.MACAddressField] = MACAddressField
NetModelSerializer.serializer_field_mapping[fields.MACAddress8Field] = MACAddress8Field



import warnings
from django.core.exceptions import FieldError
from django.db.models import Lookup, Transform, IntegerField
from django.db.models.lookups import (
    EndsWith,
    IEndsWith,
    StartsWith,
    IStartsWith,
    Regex,
    IRegex,
)
import ipaddress
from netfields.fields import InetAddressField, CidrAddressField


class InvalidLookup(Lookup):
    """
    Emulate Django 1.9 error for unsupported lookups
    """

    def as_sql(self, qn, connection):
        raise FieldError("Unsupported lookup '%s'" % self.lookup_name)


class InvalidSearchLookup(Lookup):
    """
    Emulate Django 1.9 error for unsupported search lookup
    """

    lookup_name = "search"

    def as_sql(self, qn, connection):
        raise NotImplementedError(
            "Full-text search is not implemented for this database backend"
        )


class NetFieldDecoratorMixin(object):
    def process_lhs(self, qn, connection, lhs=None):
        lhs = lhs or self.lhs
        lhs_string, lhs_params = qn.compile(lhs)
        if isinstance(
            lhs.source if hasattr(lhs, "source") else lhs.output_field, InetAddressField
        ):
            lhs_string = "HOST(%s)" % lhs_string
        elif isinstance(
            lhs.source if hasattr(lhs, "source") else lhs.output_field, CidrAddressField
        ):
            lhs_string = "TEXT(%s)" % lhs_string
        return lhs_string, list(lhs_params)


class EndsWith(NetFieldDecoratorMixin, EndsWith):
    pass


class IEndsWith(NetFieldDecoratorMixin, IEndsWith):
    pass


class StartsWith(NetFieldDecoratorMixin, StartsWith):
    pass


class IStartsWith(NetFieldDecoratorMixin, IStartsWith):
    pass


class Regex(NetFieldDecoratorMixin, Regex):
    pass


class IRegex(NetFieldDecoratorMixin, IRegex):
    pass


class NetworkLookup(object):
    def get_prep_lookup(self):
        if hasattr(self.rhs, "resolve_expression"):
            return self.rhs
        if isinstance(self.rhs, ipaddress._BaseNetwork):
            return str(self.rhs)
        return str(ipaddress.ip_network(self.rhs))


class AddressLookup(object):
    def get_prep_lookup(self):
        if hasattr(self.rhs, "resolve_expression"):
            return self.rhs
        if isinstance(self.rhs, ipaddress._BaseAddress):
            return str(self.rhs)
        return str(ipaddress.ip_interface(self.rhs))


class NetContains(AddressLookup, Lookup):
    lookup_name = "net_contains"

    def as_sql(self, qn, connection):
        lhs, lhs_params = self.process_lhs(qn, connection)
        rhs, rhs_params = self.process_rhs(qn, connection)
        params = lhs_params + rhs_params
        return "%s >> %s" % (lhs, rhs), params


class NetContained(NetworkLookup, Lookup):
    lookup_name = "net_contained"

    def as_sql(self, qn, connection):
        lhs, lhs_params = self.process_lhs(qn, connection)
        rhs, rhs_params = self.process_rhs(qn, connection)
        params = lhs_params + rhs_params
        return "%s << %s" % (lhs, rhs), params


class NetContainsOrEquals(AddressLookup, Lookup):
    lookup_name = "net_contains_or_equals"

    def as_sql(self, qn, connection):
        lhs, lhs_params = self.process_lhs(qn, connection)
        rhs, rhs_params = self.process_rhs(qn, connection)
        params = lhs_params + rhs_params
        return "%s >>= %s" % (lhs, rhs), params


class NetContainedOrEqual(NetworkLookup, Lookup):
    lookup_name = "net_contained_or_equal"

    def as_sql(self, qn, connection):
        lhs, lhs_params = self.process_lhs(qn, connection)
        rhs, rhs_params = self.process_rhs(qn, connection)
        params = lhs_params + rhs_params
        return "%s <<= %s" % (lhs, rhs), params


class NetOverlaps(NetworkLookup, Lookup):
    lookup_name = "net_overlaps"

    def as_sql(self, qn, connection):
        lhs, lhs_params = self.process_lhs(qn, connection)
        rhs, rhs_params = self.process_rhs(qn, connection)
        params = lhs_params + rhs_params
        return "%s && %s" % (lhs, rhs), params


class HostMatches(AddressLookup, Lookup):
    lookup_name = "host"

    def as_sql(self, qn, connection):
        lhs, lhs_params = self.process_lhs(qn, connection)
        rhs, rhs_params = self.process_rhs(qn, connection)
        params = lhs_params + rhs_params
        return "HOST(%s) = HOST(%s)" % (lhs, rhs), params


class Family(Transform):
    lookup_name = "family"

    def as_sql(self, compiler, connection):
        lhs, params = compiler.compile(self.lhs)
        return "family(%s)" % lhs, params

    @property
    def output_field(self):
        return IntegerField()


class _PrefixlenMixin(object):
    format_string = None

    def as_sql(self, qn, connection):
        warnings.warn(
            "min_prefixlen and max_prefixlen will be depreciated in the future; "
            "use prefixlen__gte and prefixlen__lte respectively",
            DeprecationWarning,
        )
        assert (
            self.format_string is not None
        ), "Prefixlen lookups must specify a format_string"
        lhs, lhs_params = self.process_lhs(qn, connection)
        rhs, rhs_params = self.process_rhs(qn, connection)
        params = lhs_params + rhs_params
        return self.format_string % (lhs, rhs), params

    def process_lhs(self, qn, connection, lhs=None):
        lhs = lhs or self.lhs
        lhs_string, lhs_params = qn.compile(lhs)
        lhs_string = "MASKLEN(%s)" % lhs_string
        return lhs_string, lhs_params

    def get_prep_lookup(self):
        return str(int(self.rhs))


class MaxPrefixlen(_PrefixlenMixin, Lookup):
    lookup_name = "max_prefixlen"
    format_string = "%s <= %s"


class MinPrefixlen(_PrefixlenMixin, Lookup):
    lookup_name = "min_prefixlen"
    format_string = "%s >= %s"


class Prefixlen(Transform):
    lookup_name = "prefixlen"

    def as_sql(self, compiler, connection):
        lhs, params = compiler.compile(self.lhs)
        return "masklen(%s)" % lhs, params

    @property
    def output_field(self):
        return IntegerField()



import netaddr


class mac_unix_common(netaddr.mac_eui48):
    """Common form of UNIX MAC address dialect class"""
    word_sep = ':'
    word_fmt = '%.2x'


class mac_eui64:
    """A standard IEEE EUI-64 dialect class."""

    #: The individual word size (in bits) of this address type.
    word_size = 8

    #: The number of words in this address type.
    num_words = 64 // word_size

    #: The maximum integer value for an individual word in this address type.
    max_word = 2 ** word_size - 1

    #: The separator character used between each word.
    word_sep = ":"

    #: The format string to be used when converting words to string values.
    word_fmt = "%.2x"

    #: The number base to be used when interpreting word values as integers.
    word_base = 16



from django import VERSION
from ipaddress import ip_interface, ip_network
from netaddr import EUI

from django.db.models import Case, F, When
from django.test import TestCase
from unittest import skipIf

from netfields.functions import (
    Abbrev,
    Broadcast,
    Family,
    Host,
    Hostmask,
    Macaddr8Set7bit,
    Masklen,
    Netmask,
    Network,
    SetMasklen,
    AsText,
    IsSameFamily,
    Merge,
    Trunc
)

from test.models import (
    AggregateTestChildModel,
    AggregateTestModel,
    CidrTestModel,
    InetTestModel,
    MACTestModel,
    MAC8TestModel
)


class TestInetFieldFunctions(TestCase):
    def setUp(self):
        InetTestModel.objects.create(field='10.1.0.1/16')
        InetTestModel.objects.create(field='2001:4f8:3:ba::1/64')

    def test_abbreviate(self):
        qs = InetTestModel.objects.annotate(abbrv=Abbrev(F('field')))
        self.assertEqual(qs[0].abbrv, '10.1.0.1/16')
        self.assertEqual(qs[1].abbrv, '2001:4f8:3:ba::1/64')

    def test_broadcast(self):
        qs = InetTestModel.objects.annotate(broadcast=Broadcast(F('field')))
        self.assertEqual(qs[0].broadcast, ip_interface('10.1.255.255/16'))
        self.assertEqual(qs[1].broadcast, ip_interface('2001:4f8:3:ba:ffff:ffff:ffff:ffff/64'))

    def test_family(self):
        qs = InetTestModel.objects.annotate(family=Family(F('field')))
        self.assertEqual(qs[0].family, 4)
        self.assertEqual(qs[1].family, 6)

    def test_host(self):
        qs = InetTestModel.objects.annotate(host=Host(F('field')))
        self.assertEqual(qs[0].host, '10.1.0.1')
        self.assertEqual(qs[1].host, '2001:4f8:3:ba::1')

    def test_hostmask(self):
        qs = InetTestModel.objects.annotate(hostmask=Hostmask(F('field')))
        self.assertEqual(qs[0].hostmask, ip_interface('0.0.255.255'))
        self.assertEqual(qs[1].hostmask, ip_interface('::ffff:ffff:ffff:ffff'))

    def test_masklen(self):
        qs = InetTestModel.objects.annotate(masklen=Masklen(F('field')))
        self.assertEqual(qs[0].masklen, 16)
        self.assertEqual(qs[1].masklen, 64)

    def test_netmask(self):
        qs = InetTestModel.objects.annotate(netmask=Netmask(F('field')))
        self.assertEqual(qs[0].netmask, ip_interface('255.255.0.0'))
        self.assertEqual(qs[1].netmask, ip_interface('ffff:ffff:ffff:ffff::'))

    def test_network(self):
        qs = InetTestModel.objects.annotate(network=Network(F('field')))
        self.assertEqual(qs[0].network, ip_network('10.1.0.0/16'))
        self.assertEqual(qs[1].network, ip_network('2001:4f8:3:ba::/64'))

    def test_set_masklen(self):
        (
            InetTestModel.objects
            .annotate(family=Family(F('field')))
            .update(
                field=Case(
                    When(family=4, then=SetMasklen(F('field'), 24)),
                    When(family=6, then=SetMasklen(F('field'), 120))
                )
            )
        )
        qs = InetTestModel.objects.all()
        self.assertEqual(qs[0].field, ip_interface('10.1.0.1/24'))
        self.assertEqual(qs[1].field, ip_interface('2001:4f8:3:ba::1/120'))
 
    def test_as_text(self):
        qs = InetTestModel.objects.annotate(text=AsText(F('field')))
        self.assertEqual(qs[0].text, '10.1.0.1/16')
        self.assertEqual(qs[1].text, '2001:4f8:3:ba::1/64')

    def test_is_same_family(self):
        parent = AggregateTestModel.objects.create(inet='0.0.0.0/0')
        AggregateTestChildModel.objects.create(
            parent=parent, inet='10.1.0.1/16', network='10.1.0.0/16'
        )
        AggregateTestChildModel.objects.create(
            parent=parent, inet='2001:4f8:3:ba::1/64', network='2001:4f8:3:ba::/64'
        )

        qs = (
            AggregateTestChildModel.objects.annotate(
                is_same_family=IsSameFamily(F('inet'), F('parent__inet'))
            )
            .order_by('id')
        )
        self.assertEqual(qs[0].is_same_family, True)
        self.assertEqual(qs[1].is_same_family, False)

    def test_merge(self):
        parent = AggregateTestModel.objects.create(inet='10.0.0.0/24')
        AggregateTestChildModel.objects.create(
            parent=parent, inet='10.0.1.0/24', network='10.0.0.0/23'
        )

        parent = AggregateTestModel.objects.create(inet='2001:4f8:3:ba::/64')
        AggregateTestChildModel.objects.create(
            parent=parent, inet='2001:4f8:3:bb::/64', network='2001:4f8:3:ba::/63'
        )

        qs = (
            AggregateTestChildModel.objects.annotate(
                merged=Merge(F('inet'), F('parent__inet'))
            )
        )
        self.assertEqual(qs[0].merged, qs[0].network)
        self.assertEqual(qs[1].merged, qs[1].network)


class TestCidrFieldFunctions(TestCase):
    def setUp(self):
        CidrTestModel.objects.create(field='10.1.0.0/16')
        CidrTestModel.objects.create(field='2001:4f8:3:ba::/64')

    def test_abbreviate(self):
        qs = CidrTestModel.objects.annotate(abbrv=Abbrev(F('field')))
        self.assertEqual(qs[0].abbrv, '10.1/16')
        self.assertEqual(qs[1].abbrv, '2001:4f8:3:ba/64')

    def test_broadcast(self):
        qs = CidrTestModel.objects.annotate(broadcast=Broadcast(F('field')))
        self.assertEqual(qs[0].broadcast, ip_interface('10.1.255.255/16'))
        self.assertEqual(qs[1].broadcast, ip_interface('2001:4f8:3:ba:ffff:ffff:ffff:ffff/64'))

    def test_family(self):
        qs = CidrTestModel.objects.annotate(family=Family(F('field')))
        self.assertEqual(qs[0].family, 4)
        self.assertEqual(qs[1].family, 6)

    def test_host(self):
        qs = CidrTestModel.objects.annotate(host=Host(F('field')))
        self.assertEqual(qs[0].host, '10.1.0.0')
        self.assertEqual(qs[1].host, '2001:4f8:3:ba::')

    def test_hostmask(self):
        qs = CidrTestModel.objects.annotate(hostmask=Hostmask(F('field')))
        self.assertEqual(qs[0].hostmask, ip_interface('0.0.255.255'))
        self.assertEqual(qs[1].hostmask, ip_interface('::ffff:ffff:ffff:ffff'))

    def test_masklen(self):
        qs = CidrTestModel.objects.annotate(masklen=Masklen(F('field')))
        self.assertEqual(qs[0].masklen, 16)
        self.assertEqual(qs[1].masklen, 64)

    def test_netmask(self):
        qs = CidrTestModel.objects.annotate(netmask=Netmask(F('field')))
        self.assertEqual(qs[0].netmask, ip_interface('255.255.0.0'))
        self.assertEqual(qs[1].netmask, ip_interface('ffff:ffff:ffff:ffff::'))

    def test_network(self):
        qs = CidrTestModel.objects.annotate(network=Network(F('field')))
        self.assertEqual(qs[0].network, ip_network('10.1.0.0/16'))
        self.assertEqual(qs[1].network, ip_network('2001:4f8:3:ba::/64'))

    def test_set_masklen(self):
        (
            CidrTestModel.objects
            .annotate(family=Family(F('field')))
            .update(
                field=Case(
                    When(family=4, then=SetMasklen(F('field'), 24)),
                    When(family=6, then=SetMasklen(F('field'), 120))
                )
            )
        )
        qs = CidrTestModel.objects.all()
        self.assertEqual(qs[0].field, ip_network('10.1.0.0/24'))
        self.assertEqual(qs[1].field, ip_network('2001:4f8:3:ba::/120'))

    def test_as_text(self):
        qs = CidrTestModel.objects.annotate(text=AsText(F('field')))
        self.assertEqual(qs[0].text, '10.1.0.0/16')
        self.assertEqual(qs[1].text, '2001:4f8:3:ba::/64')

    def test_is_same_family(self):
        parent = AggregateTestModel.objects.create(network='0.0.0.0/0')
        AggregateTestChildModel.objects.create(
            parent=parent, inet= '10.1.0.1/16', network='10.1.0.0/16'
        )
        AggregateTestChildModel.objects.create(
            parent=parent, inet='2001:4f8:3:ba::1/64', network='2001:4f8:3:ba::/64'
        )

        qs = (
            AggregateTestChildModel.objects.annotate(
                is_same_family=IsSameFamily(F('network'), F('parent__network'))
            )
            .order_by('id')
        )
        self.assertEqual(qs[0].is_same_family, True)
        self.assertEqual(qs[1].is_same_family, False)

    def test_merge(self):
        parent = AggregateTestModel.objects.create(network='10.0.0.0/24')
        AggregateTestChildModel.objects.create(
            parent=parent, inet='10.0.1.0/24', network='10.0.0.0/23'
        )

        parent = AggregateTestModel.objects.create(network='2001:4f8:3:ba::/64')
        AggregateTestChildModel.objects.create(
            parent=parent, inet='2001:4f8:3:bb::/64', network='2001:4f8:3:ba::/63'
        )

        qs = (
            AggregateTestChildModel.objects.annotate(
                merged=Merge(F('network'), F('parent__network'))
            )
        )
        self.assertEqual(qs[0].merged, qs[0].network)
        self.assertEqual(qs[1].merged, qs[1].network)

    @skipIf(VERSION < (2, 0), 'Django unable to resolve type of num_ips to be IntegerField until 2.0.')
    def test_read_me_example(self):
        qs = (
            CidrTestModel.objects.annotate(
                family=Family(F('field')),
                num_ips=2 ** (32 - Masklen(F('field'))),
            )
            .filter(family=4)
        )
        self.assertEqual(qs[0].num_ips, 65536)


class TestMacFieldFunctions(TestCase):
    def setUp(self):
        MACTestModel.objects.create(field='88:bb:cc:dd:ee:ff')

    def test_trunc(self):
        qs = MACTestModel.objects.annotate(trunc=Trunc(F('field')))
        self.assertEqual(qs[0].trunc, EUI('88:bb:cc:00:00:00'))

    def test_macaddr8_to7bit(self):
        qs = MACTestModel.objects.annotate(eui64=Macaddr8Set7bit(F('field')))
        self.assertEqual(qs[0].eui64, EUI('8a:bb:cc:ff:fe:dd:ee:ff'))


class TestMac8FieldFunctions(TestCase):
    def setUp(self):
        MAC8TestModel.objects.create(field='88:99:aa:bb:cc:dd:ee:ff')

    def test_trunc(self):
        qs = MAC8TestModel.objects.annotate(trunc=Trunc(F('field')))
        self.assertEqual(qs[0].trunc, EUI('88:99:aa:00:00:00:00:00'))

    def test_macaddr8_to7bit(self):
        qs = MAC8TestModel.objects.annotate(eui64=Macaddr8Set7bit(F('field')))
        self.assertEqual(qs[0].eui64, EUI('8a:99:aa:bb:cc:dd:ee:ff'))



from ipaddress import ip_address, ip_interface, ip_network
from netaddr import EUI

from django.forms import ModelForm
from django.test import TestCase

from test.models import (
    CidrTestModel,
    InetTestModel,
    UniqueInetTestModel,
    UniqueCidrTestModel,
    NoPrefixInetTestModel,
    MACTestModel,
    MAC8TestModel
)
from netfields.mac import mac_eui64, mac_unix_common


class InetAddressTestModelForm(ModelForm):
    class Meta:
        model = InetTestModel
        exclude = []


class TestInetAddressFormField(TestCase):
    form_class = InetAddressTestModelForm

    def test_form_ipv4_valid(self):
        form = self.form_class({'field': '10.0.0.1'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], ip_interface('10.0.0.1'))

    def test_form_ipv4_invalid(self):
        form = self.form_class({'field': '10.0.0.1.2'})
        self.assertFalse(form.is_valid())

    def test_form_ipv4_strip(self):
        form = self.form_class({'field': ' 10.0.0.1 '})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], ip_interface('10.0.0.1'))

    def test_form_ipv4_change(self):
        instance = InetTestModel.objects.create(field='10.1.2.3/24')
        form = self.form_class({'field': '10.1.2.4/24'}, instance=instance)
        self.assertTrue(form.is_valid())
        form.save()
        instance = InetTestModel.objects.get(pk=instance.pk)
        self.assertEqual(instance.field, ip_interface('10.1.2.4/24'))

    def test_form_ipv6_valid(self):
        form = self.form_class({'field': '2001:0:1::2'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], ip_interface('2001:0:1::2'))

    def test_form_ipv6_invalid(self):
        form = self.form_class({'field': '2001:0::1::2'})
        self.assertFalse(form.is_valid())

    def test_form_ipv6_strip(self):
        form = self.form_class({'field': ' 2001:0:1::2 '})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], ip_interface('2001:0:1::2'))

    def test_form_ipv6_change(self):
        instance = InetTestModel.objects.create(field='2001:0:1::2/64')
        form = self.form_class({'field': '2001:0:1::3/64'}, instance=instance)
        self.assertTrue(form.is_valid())
        form.save()
        instance = InetTestModel.objects.get(pk=instance.pk)
        self.assertEqual(instance.field, ip_interface('2001:0:1::3/64'))


class NoPrefixInetAddressTestModelForm(ModelForm):
    class Meta:
        model = NoPrefixInetTestModel
        exclude = []


class TestNoPrefixInetAddressFormField(TestCase):
    form_class = NoPrefixInetAddressTestModelForm

    def test_form_ipv4_valid(self):
        form = self.form_class({'field': '10.0.0.1'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], ip_address('10.0.0.1'))

    def test_form_ipv4_invalid(self):
        form = self.form_class({'field': '10.0.0.1.2'})
        self.assertFalse(form.is_valid())

    def test_form_ipv4_prefix_invalid(self):
        instance = NoPrefixInetTestModel.objects.create(field='10.1.2.3/24')
        form = self.form_class({'field': '10.1.2.4/24'}, instance=instance)
        self.assertFalse(form.is_valid())

    def test_form_ipv6_valid(self):
        form = self.form_class({'field': '2001:0:1::2'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], ip_address('2001:0:1::2'))

    def test_form_ipv6_invalid(self):
        form = self.form_class({'field': '2001:0::1::2'})
        self.assertFalse(form.is_valid())

    def test_form_ipv6_prefix_invalid(self):
        instance = NoPrefixInetTestModel.objects.create(field='2001:0:1::2/64')
        form = self.form_class({'field': '2001:0:1::3/64'}, instance=instance)
        self.assertFalse(form.is_valid())


class UniqueInetAddressTestModelForm(ModelForm):
    class Meta:
        model = UniqueInetTestModel
        exclude = []


class TestUniqueInetAddressFormField(TestInetAddressFormField):
    form_class = UniqueInetAddressTestModelForm


class CidrAddressTestModelForm(ModelForm):
    class Meta:
        model = CidrTestModel
        exclude = []


class TestCidrAddressFormField(TestCase):
    form_class = CidrAddressTestModelForm

    def test_form_ipv4_valid(self):
        form = self.form_class({'field': '10.0.1.0/24'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], ip_network('10.0.1.0/24'))

    def test_form_ipv4_invalid(self):
        form = self.form_class({'field': '10.0.0.1.2/32'})
        self.assertFalse(form.is_valid())

    def test_form_ipv4_strip(self):
        form = self.form_class({'field': ' 10.0.1.0/24 '})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], ip_network('10.0.1.0/24'))

    def test_form_ipv4_bits_to_right_of_mask(self):
        form = self.form_class({'field': '10.0.0.1.2/24'})
        self.assertFalse(form.is_valid())

    def test_form_ipv6_valid(self):
        form = self.form_class({'field': '2001:0:1::/64'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], ip_network('2001:0:1::/64'))

    def test_form_ipv6_invalid(self):
        form = self.form_class({'field': '2001:0::1::2/128'})
        self.assertFalse(form.is_valid())

    def test_form_ipv6_strip(self):
        form = self.form_class({'field': ' 2001:0:1::/64 '})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], ip_network('2001:0:1::/64'))

    def test_form_ipv6_bits_to_right_of_mask(self):
        form = self.form_class({'field': '2001:0::1::2/64'})
        self.assertFalse(form.is_valid())


class UniqueCidrAddressTestModelForm(ModelForm):
    class Meta:
        model = UniqueCidrTestModel
        exclude = []


class TestUniqueCidrAddressFormField(TestCidrAddressFormField):
    form_class = UniqueCidrAddressTestModelForm


class MacAddressTestModelForm(ModelForm):
    class Meta:
        model = MACTestModel
        exclude = []


class TestMacAddressFormField(TestCase):
    def setUp(self):
        self.mac = EUI('00:aa:2b:c3:dd:44', dialect=mac_unix_common)

    def test_unix(self):
        form = MacAddressTestModelForm({'field': '0:AA:2b:c3:dd:44'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], self.mac)

    def test_unix_common(self):
        form = MacAddressTestModelForm({'field': '00:aa:2b:c3:dd:44'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], self.mac)

    def test_eui48(self):
        form = MacAddressTestModelForm({'field': '00-AA-2B-C3-DD-44'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], self.mac)

    def test_cisco(self):
        form = MacAddressTestModelForm({'field': '00aa.2bc3.dd44'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], self.mac)

    def test_24bit_colon(self):
        form = MacAddressTestModelForm({'field': '00aa2b:c3dd44'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], self.mac)

    def test_24bit_hyphen(self):
        form = MacAddressTestModelForm({'field': '00aa2b-c3dd44'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], self.mac)

    def test_bare(self):
        form = MacAddressTestModelForm({'field': '00aa2b:c3dd44'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], self.mac)

    def test_strip(self):
        form = MacAddressTestModelForm({'field': ' 00:aa:2b:c3:dd:44 '})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], self.mac)

    def test_eui64(self):
        form = MacAddressTestModelForm({'field': '00-AA-2B-C3-DD-44-EE-55'})
        self.assertFalse(form.is_valid())

    def test_invalid(self):
        form = MacAddressTestModelForm({'field': 'notvalid'})
        self.assertFalse(form.is_valid())


class MacAddress8TestModelForm(ModelForm):
    class Meta:
        model = MAC8TestModel
        exclude = []


class TestMacAddress8FormField(TestCase):
    def setUp(self):
        self.mac = EUI('00:aa:2b:c3:dd:44:55:ff', dialect=mac_eui64)

    def test_unix(self):
        form = MacAddress8TestModelForm({'field': '0:AA:2b:c3:dd:44:55:FF'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], self.mac)

    def test_unix_common(self):
        form = MacAddress8TestModelForm({'field': '00:aa:2b:c3:dd:44:55:ff'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], self.mac)

    def test_eui64(self):
        form = MacAddress8TestModelForm({'field': '00-AA-2B-C3-DD-44-55-FF'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], self.mac)

    def test_cisco(self):
        form = MacAddress8TestModelForm({'field': '00aa.2bc3.dd44.55ff'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], self.mac)

    def test_16bit_colon(self):
        form = MacAddress8TestModelForm({'field': '00aa:2bc3:dd44:55ff'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], self.mac)

    def test_16bit_hyphen(self):
        form = MacAddress8TestModelForm({'field': '00aa-2bc3-dd44-55ff'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], self.mac)

    def test_strip(self):
        form = MacAddress8TestModelForm({'field': ' 00:aa:2b:c3:dd:44:55:ff '})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], self.mac)

    def test_eui48(self):
        form = MacAddress8TestModelForm({'field': '00:aa:2b:c3:dd:44'})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field'], EUI('00:aa:2b:ff:fe:c3:dd:44', dialect=mac_eui64))

    def test_invalid(self):
        form = MacAddress8TestModelForm({'field': 'notvalid'})
        self.assertFalse(form.is_valid())



from django import VERSION

if VERSION[0] < 2:
    from django.db.backends.postgresql_psycopg2.base import DatabaseWrapper
else:
    from django.db.backends.postgresql.base import DatabaseWrapper

try:
    from django.db.backends.postgresql.base import is_psycopg3
except ImportError:
    is_psycopg3 = False



from django import VERSION
from django.contrib.postgres.fields import ArrayField
from django.db.models import CASCADE, ForeignKey, Model

from netfields import (
    CidrAddressField,
    InetAddressField,
    MACAddress8Field,
    MACAddressField,
    NetManager,
)


class InetTestModel(Model):
    field = InetAddressField()
    objects = NetManager()

    class Meta:
        db_table = "inet"


class NullInetTestModel(Model):
    field = InetAddressField(null=True)
    objects = NetManager()

    class Meta:
        db_table = "nullinet"


class UniqueInetTestModel(Model):
    field = InetAddressField(unique=True)
    objects = NetManager()

    class Meta:
        db_table = "uniqueinet"


class NoPrefixInetTestModel(Model):
    field = InetAddressField(store_prefix_length=False)
    objects = NetManager()

    class Meta:
        db_table = "noprefixinet"


class CidrTestModel(Model):
    field = CidrAddressField()
    objects = NetManager()

    class Meta:
        db_table = "cidr"


class NullCidrTestModel(Model):
    field = CidrAddressField(null=True)
    objects = NetManager()

    class Meta:
        db_table = "nullcidr"


class UniqueCidrTestModel(Model):
    field = CidrAddressField(unique=True)
    objects = NetManager()

    class Meta:
        db_table = "uniquecidr"


class MACTestModel(Model):
    field = MACAddressField(null=True)
    objects = NetManager()

    class Meta:
        db_table = "mac"


class MAC8TestModel(Model):
    field = MACAddress8Field(null=True)
    objects = NetManager()

    class Meta:
        db_table = "mac8"


class InetArrayTestModel(Model):
    field = ArrayField(InetAddressField(), blank=True, null=True)

    class Meta:
        db_table = "inetarray"


class CidrArrayTestModel(Model):
    field = ArrayField(CidrAddressField(), blank=True, null=True)

    class Meta:
        db_table = "cidrarray"


class MACArrayTestModel(Model):
    field = ArrayField(MACAddressField(), blank=True, null=True)

    class Meta:
        db_table = "macarray"


class MAC8ArrayTestModel(Model):
    field = ArrayField(MACAddress8Field(), blank=True, null=True)

    class Meta:
        db_table = "mac8array"


class AggregateTestModel(Model):
    network = CidrAddressField(blank=True, null=True, default=None)
    inet = InetAddressField(blank=True, null=True, default=None)


class AggregateTestChildModel(Model):
    parent = ForeignKey(
        "AggregateTestModel",
        related_name="children",
        on_delete=CASCADE,
    )
    network = CidrAddressField()
    inet = InetAddressField()


if VERSION >= (5, 1):
    from django.db.models import F, Q, CheckConstraint

    class ConstraintModel(Model):
        network = CidrAddressField()
        inet = InetAddressField()

        class Meta:
            constraints = (
                CheckConstraint(
                    condition=Q(network__net_contains=F("inet")),
                    name="inet_contained",
                ),
            )

elif VERSION >= (4, 1):
    from django.db.models import F, Q, CheckConstraint

    class ConstraintModel(Model):
        network = CidrAddressField()
        inet = InetAddressField()

        class Meta:
            constraints = (
                CheckConstraint(
                    check=Q(network__net_contains=F("inet")),
                    name="inet_contained",
                ),
            )



from django import VERSION

from netfields.managers import NetManager
from netfields.fields import (InetAddressField, CidrAddressField,
                              MACAddressField, MACAddress8Field)

# only keep it for django 3.1 and below
if VERSION[0] < 3 or VERSION[0] == 3 and VERSION[1]  < 2:
    default_app_config = 'netfields.apps.NetfieldsConfig'



from django.db import models
from ipaddress import _BaseNetwork

try:
    str_type = unicode
except NameError:
    str_type = str


class NetManager(models.Manager):
    use_for_related_fields = True

    def filter(self, *args, **kwargs):
        for key, val in kwargs.items():
            if isinstance(val, _BaseNetwork):
                # Django will attempt to consume the _BaseNetwork iterator, which
                # will convert it to a list of every address in the network
                kwargs[key] = str_type(val)
        return super(NetManager, self).filter(*args, **kwargs)



from psycopg.adapt import Dumper, Loader
from psycopg.postgres import adapters


class Inet(str):
    def __repr__(self):
        return '%s(%s)' % (self.__class__.__name__, self)


class Macaddr(Inet):
    pass


class Macaddr8(Inet):
    pass


class _MacaddrDumper(Dumper):
    oid = adapters.types['macaddr'].oid

    def dump(self, obj):
        return str(obj).encode()


class _MacaddrLoader(Loader):
    def load(self, data):
        if isinstance(data, memoryview):
            data = bytes(data)
        return Macaddr(data.decode())


class _Macaddr8Dumper(Dumper):
    oid = adapters.types['macaddr8'].oid

    def dump(self, obj):
        return str(obj).encode()


class _Macaddr8Loader(Loader):
    def load(self, data):
        if isinstance(data, memoryview):
            data = bytes(data)
        return Macaddr8(data.decode())


adapters.register_loader('macaddr', _MacaddrLoader)
adapters.register_loader('macaddr8', _Macaddr8Loader)
adapters.register_dumper(Macaddr, _MacaddrDumper)
adapters.register_dumper(Macaddr8, _Macaddr8Dumper)
